use std::path::PathBuf;

use clap::Parser;

/// read and return command line arguments or exit on invalid arguments
pub fn parse() -> Args {
    Args::parse()
}

/// Sopra Einzelprojekt der UULM, 2024 - Mark B
#[derive(Parser)]
#[clap(version)]
pub struct Args {
    /// The path to the config file.
    ///
    /// Defaults to `starfighter_alliance_config.json`.
    /// The highscores will be saved in `<config-path-with-extension>.highscores.toml`
    #[clap(long)]
    pub config: Option<PathBuf>,
    /// Enable spec-compliant obstacle spawning by default.
    ///
    /// This causes obstacles to spawn in waves, where all obstacles
    /// of one wave are spawned at the same height and move with the same speed.
    /// It can also be enabled in the options, but this is not saved when the game exits.
    #[clap(long)]
    pub disallow_fun: bool,
}
