use asciivation_duty_renderer::CheckRes;

use crate::parts::{main_menu::MainMenu, AnyPart};

use crate::tests::*;

#[test]
fn keyboard_enhancement_protocol_popup() {
    MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: false,
        })
        .check(|r| {
            assert!(
                r.state.popups.has_popup(),
                "popup should be displayed because keyboard enhacement protocol is not supported"
            )
        });
}

#[test]
fn keyboard_enhancement_protocol_no_popup() {
    MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: true,
        })
        .check(|r| {
            assert!(
                !r.state.popups.has_popup(),
                "no popup should be displayed because keyboard enhacement protocol is supported"
            )
        });
}

#[test]
fn exit_on_esc() {
    MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: true,
        })
        .event(Event::Key(KeyEvent {
            code: KeyCode::Esc,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect_err("Should exit after Esc was pressed directly after the game launched");
}

#[test]
fn exit_button() {
    MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: true,
        })
        .event(Event::Key(KeyEvent {
            code: KeyCode::Left,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect("Should not exit after a Left event")
        .event(Event::Key(KeyEvent {
            code: KeyCode::Left,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect("Should not exit after two Left events")
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect_err("Should exit after the enter event");
}

#[test]
fn options_button() {
    match MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: true,
        })
        .event(Event::Key(KeyEvent {
            code: KeyCode::Left,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect("Should not exit after a Left event")
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect_err("Should leave main menu after the enter event")
        .expect("Should not exit after the enter event, but go to a different screen")
    {
        AnyPart::OptionsScreen(_) => (),
        part => panic!("Expected an OptionsScreen, but got {part:?}"),
    }
}

#[test]
fn game_button() {
    match MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: true,
        })
        .event(Event::Key(KeyEvent {
            code: KeyCode::Right,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect("Should not exit after a Right event")
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect_err("Should leave main menu after the enter event")
        .expect("Should not exit after the enter event, but go to a different screen")
    {
        AnyPart::GameState(_) => (),
        part => panic!("Expected a GameState, but got {part:?}"),
    }
}

#[test]
fn spaceship_button() {
    match MainMenu::new()
        .test(&CheckRes {
            size: (80, 30),
            supports_keyboard_enhancement: true,
        })
        .event(Event::Key(KeyEvent {
            code: KeyCode::Right,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect("Should not exit after a Right event")
        .event(Event::Key(KeyEvent {
            code: KeyCode::Right,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect("Should not exit after a second Right event")
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .expect_err("Should leave main menu after the enter event")
        .expect("Should not exit after the enter event, but go to a different screen")
    {
        AnyPart::SpaceshipSelectionScreen(_) => (),
        part => panic!("Expected a SpaceshipSelectionScreen, but got {part:?}"),
    }
}
