pub mod list;
pub mod screen;
pub mod ship;

const CHAR_HEALTH: char = 'H';
const CHAR_SHOT_POWER: char = 'P';
const CHAR_SHOT_RATE: char = 'R';
const CHAR_SPEED: char = 'V';
const CLR_HEALTH: Color = Color::rgb(255, 0, 200);
const CLR_SHOT_POWER: Color = Color::rgb(255, 100, 100);
const CLR_SHOT_RATE: Color = Color::rgb(255, 150, 0);
const CLR_SPEED: Color = Color::rgb(150, 150, 255);

const COLOR_NEUTRAL: Color = Color::rgb(170, 170, 170);
const COLOR_BETTER: Color = Color::rgb(155, 200, 155);
const COLOR_WORSE: Color = Color::rgb(80, 0, 0);

use asciivation_duty_renderer::render::{Color, Pixel, CLR_BLACK};
pub use screen::SpaceshipSelectionScreen;

#[allow(clippy::too_many_arguments)]
fn draw_stats_bar(
    #[allow(clippy::ptr_arg)] screen_buf: &mut Vec<Vec<Pixel>>,
    col: usize,
    stats_bars_base: usize,
    stats_bars_height: usize,
    val: f32,
    min: f32,
    max: f32,
    cmp: Option<f32>,
) {
    let mut val_rel = (val - min) / (max - min);
    if !val_rel.is_finite() {
        val_rel = 0.0;
    }
    val_rel = val_rel.clamp(0.0, 1.0);
    let val8 = (val_rel * 8.0 * stats_bars_height as f32).round() as usize;
    let val1 = val8 / 8;
    let valp = val8 % 8;
    if let Some(cmp) = cmp.filter(|cmp| *cmp != val) {
        let mut cmp_rel = (cmp - min) / (max - min);
        if !cmp_rel.is_finite() {
            cmp_rel = 0.0;
        }
        cmp_rel = cmp_rel.clamp(0.0, 1.0);
        let cmp8 = (cmp_rel * 8.0 * stats_bars_height as f32).round() as usize;
        let cmp1 = cmp8 / 8;
        let cmpp = cmp8 % 8;
        if cmp > val {
            for y in 0..val1 {
                screen_buf[stats_bars_base - y][col] = Pixel {
                    bg: COLOR_NEUTRAL,
                    fg: COLOR_NEUTRAL,
                    char: ' ',
                }
            }
            for y in val1..cmp1 {
                screen_buf[stats_bars_base - y][col] = Pixel {
                    bg: COLOR_WORSE,
                    fg: COLOR_WORSE,
                    char: ' ',
                }
            }
            if valp > 0 {
                screen_buf[stats_bars_base - val1][col] = Pixel {
                    bg: COLOR_WORSE,
                    fg: COLOR_NEUTRAL,
                    char: ['▁', '▂', '▃', '▄', '▅', '▆', '▇'][valp - 1],
                };
            }
            if cmpp > 0 {
                screen_buf[stats_bars_base - cmp1][col] = Pixel {
                    bg: CLR_BLACK,
                    fg: COLOR_WORSE,
                    char: ['▁', '▂', '▃', '▄', '▅', '▆', '▇'][cmpp - 1],
                };
            }
        } else {
            for y in 0..cmp1 {
                screen_buf[stats_bars_base - y][col] = Pixel {
                    bg: COLOR_NEUTRAL,
                    fg: COLOR_NEUTRAL,
                    char: ' ',
                }
            }
            for y in cmp1..val1 {
                screen_buf[stats_bars_base - y][col] = Pixel {
                    bg: COLOR_BETTER,
                    fg: COLOR_BETTER,
                    char: ' ',
                }
            }
            if cmpp > 0 {
                screen_buf[stats_bars_base - cmp1][col] = Pixel {
                    bg: COLOR_BETTER,
                    fg: COLOR_NEUTRAL,
                    char: ['▁', '▂', '▃', '▄', '▅', '▆', '▇'][cmpp - 1],
                };
            }
            if valp > 0 {
                screen_buf[stats_bars_base - val1][col] = Pixel {
                    bg: CLR_BLACK,
                    fg: COLOR_BETTER,
                    char: ['▁', '▂', '▃', '▄', '▅', '▆', '▇'][valp - 1],
                };
            }
        }
    } else {
        for y in 0..val1 {
            screen_buf[stats_bars_base - y][col] = Pixel {
                bg: COLOR_NEUTRAL,
                fg: COLOR_NEUTRAL,
                char: ' ',
            }
        }
        if valp > 0 {
            screen_buf[stats_bars_base - val1][col] = Pixel {
                bg: CLR_BLACK,
                fg: COLOR_NEUTRAL,
                char: ['▁', '▂', '▃', '▄', '▅', '▆', '▇'][valp - 1],
            };
        }
    }
}
