use std::time::Duration;

use crate::parts::main_menu::MainMenu;

use super::*;

#[test]
fn highscore_added_after_game() {
    // start with a main menu and switch to the game screen from there
    eprintln!("Now in main menu");
    let mut main_menu = MainMenu::new().test(&CheckRes {
        size: (80, 30),
        supports_keyboard_enhancement: true,
    });
    let mut scores_count = usize::MAX - 1;
    let main_menu_end = main_menu
        // start a game (Right -> Enter)
        .draw()
        .unwrap()
        .tick(Duration::from_secs(3))
        .draw()
        .unwrap()
        .check(|r| {
            scores_count = r.state.highscores.scores().len();
        })
        .event(Event::Key(KeyEvent {
            code: KeyCode::Right,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .draw()
        .unwrap()
        .event(Event::Key(KeyEvent {
            code: KeyCode::Right,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Release,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .tick(Duration::from_secs(1))
        .draw()
        .unwrap()
        .tick(Duration::from_secs(1))
        .draw()
        .unwrap()
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .and_then(|v| {
            v.tick(Duration::from_secs(1)).draw().and_then(|v| {
                v.tick(Duration::from_secs(1))
                    .event(Event::Key(KeyEvent {
                        code: KeyCode::Enter,
                        modifiers: KeyModifiers::empty(),
                        kind: KeyEventKind::Release,
                        state: KeyEventState::empty(),
                    }))
                    .unwrap()
                    .tick(Duration::from_secs(1))
                    .draw()
                    .and_then(|v| v.tick(Duration::from_secs(1)).draw())
            })
        })
        .expect_err("should transition to a different screen now");
    let mut game_screen = if let Some(AnyPart::GameState(game_screen)) = main_menu_end {
        game_screen.test_from(main_menu)
    } else {
        panic!("should transition to the game screen now, but end was {main_menu_end:?}");
    };
    eprintln!("Now in game screen");
    // wait a few seconds, then use the options menu to exit the game, which should transition to the end screen
    let game_screen_end = game_screen
        .draw()
        .unwrap()
        .tick(Duration::from_secs(5))
        .draw()
        .unwrap()
        .event(Event::Key(KeyEvent {
            code: KeyCode::Esc,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .draw()
        .unwrap()
        .tick(Duration::from_millis(100))
        .event(Event::Key(KeyEvent {
            code: KeyCode::Esc,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Release,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .draw()
        .unwrap()
        .tick(Duration::from_millis(100))
        .event(Event::Key(KeyEvent {
            code: KeyCode::Down,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .draw()
        .unwrap()
        .tick(Duration::from_millis(100))
        .event(Event::Key(KeyEvent {
            code: KeyCode::Down,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Release,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .draw()
        .unwrap()
        .tick(Duration::from_millis(100))
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .unwrap()
        .draw()
        .and_then(|v| {
            v.tick(Duration::from_millis(100))
                .event(Event::Key(KeyEvent {
                    code: KeyCode::Enter,
                    modifiers: KeyModifiers::empty(),
                    kind: KeyEventKind::Release,
                    state: KeyEventState::empty(),
                }))
                .and_then(|v| {
                    v.draw()
                        .and_then(|v| v.tick(Duration::from_millis(100)).draw())
                })
        })
        .expect_err("should transition to end screen now");
    let mut end_screen = if let AnyPart::EndScreen(end_screen) = game_screen_end {
        end_screen.test_from(game_screen)
    } else {
        panic!("should transition to the end screen now, but end was {game_screen_end:?}");
    };
    eprintln!("Now in end screen");
    for ch in "test dummy".chars() {
        end_screen
            .event(Event::Key(KeyEvent {
                code: KeyCode::Char(ch),
                modifiers: KeyModifiers::empty(),
                kind: KeyEventKind::Press,
                state: KeyEventState::empty(),
            }))
            .unwrap()
            .event(Event::Key(KeyEvent {
                code: KeyCode::Char(ch),
                modifiers: KeyModifiers::empty(),
                kind: KeyEventKind::Release,
                state: KeyEventState::empty(),
            }))
            .unwrap();
    }
    end_screen
        .event(Event::Key(KeyEvent {
            code: KeyCode::Enter,
            modifiers: KeyModifiers::empty(),
            kind: KeyEventKind::Press,
            state: KeyEventState::empty(),
        }))
        .and_then(|v| {
            v.event(Event::Key(KeyEvent {
                code: KeyCode::Enter,
                modifiers: KeyModifiers::empty(),
                kind: KeyEventKind::Release,
                state: KeyEventState::empty(),
            }))
            .and_then(|v| v.draw().and_then(|v| v.tick(Duration::from_secs(1)).draw()))
        })
        .expect_err("should exit the end screen now");
    end_screen.check(|r| {
        assert!(
            r.state.highscores.scores().len() > scores_count,
            "there should be a new score now, but score count was {} and is now {}",
            scores_count,
            r.state.highscores.scores().len()
        )
    });
}
